/*
 LED is a serial port communication, The settings are always 8 data bits, no parity, 1 stop bit, no start bits, 115200 baudrate.
        We is using "/dev/ttyS1".
        Only need to send a character encoding in serial port (Also can be a string).

        char String[] = {0X48,0X65,0X6C,0X6C,0X6F}; 
       Uart_send(fd, String, sizof(String));
       or
       char String[] = "Hello"
       Uart_send(fd, String, sizof(String));

       But each send a character must wait 500ms.
       A line can only display 16 characters.
        '0x0d' equal '\n' is a newline. (after sending wait 2500us)
        '0xff' is remove all show.
*/
#include<stdio.h>
#include<stdlib.h>
#include<unistd.h>
#include<sys/types.h>
#include<sys/stat.h>
#include<sys/signal.h>
#include<fcntl.h>
#include<termios.h>
#include<errno.h>
#include<string.h>

#define FALSE -1
#define TRUE 0
#define flag 1
#define noflag 0

int wait_flag = noflag;
int STOP = 0;
int res;

int speed_arr[] = {B115200, B38400, B19200, B9600, B4800, B2400, B1200, B300,B38400, B19200, B9600, B4800, B2400, B1200, B300};
int name_arr[] = {115200, 38400,  19200,  9600,  4800,  2400,  1200,  300, 38400, 19200,  9600, 4800, 2400, 1200,  300 };
void
set_speed (int fd, int speed)
{
  int i;
  int status;
  struct termios Opt;
  tcgetattr (fd, &Opt);
  for (i = 0; i < sizeof (speed_arr) / sizeof (int); i++)
    {
      if (speed == name_arr[i])
	{
	  tcflush (fd, TCIOFLUSH);
	  cfsetispeed (&Opt, speed_arr[i]);
	  cfsetospeed (&Opt, speed_arr[i]);
	  status = tcsetattr (fd, TCSANOW, &Opt);
	  if (status != 0)
	    {
	      perror ("tcsetattr fd1");
	      return;
	    }
	  tcflush (fd, TCIOFLUSH);
	}
    }
}

int
set_Parity (int fd, int databits, int stopbits, int parity)
{
  struct termios options;
  if (tcgetattr (fd, &options) != 0)
    {
      perror ("SetupSerial 1");
      return (FALSE);
    }
  options.c_cflag &= ~CSIZE;
  options.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
  options.c_oflag &= ~OPOST;
  switch (databits)
    {
    case 7:
      options.c_cflag |= CS7;
      break;
    case 8:
      options.c_cflag |= CS8;
      break;
    default:
      fprintf (stderr, "Unsupported data size\n");
      return (FALSE);
    }
  switch (parity)
    {
    case 'n':
    case 'N':
      options.c_cflag &= ~PARENB;	/* Clear parity enable */
      options.c_iflag &= ~INPCK;	/* Enable parity checking */
      break;
    case 'o':
    case 'O':
      options.c_cflag |= (PARODD | PARENB);
      options.c_iflag |= INPCK;	/* Disnable parity checking */
      break;
    case 'e':
    case 'E':
      options.c_cflag |= PARENB;	/* Enable parity */
      options.c_cflag &= ~PARODD;
      options.c_iflag |= INPCK;	/* Disnable parity checking */
      break;
    case 'S':
    case 's':			/*as no parity */
      options.c_cflag &= ~PARENB;
      options.c_cflag &= ~CSTOPB;
      break;
    default:
      fprintf (stderr, "Unsupported parity\n");
      return (FALSE);
    }

  switch (stopbits)
    {
    case 1:
      options.c_cflag &= ~CSTOPB;
      break;
    case 2:
      options.c_cflag |= CSTOPB;
      break;
    default:
      fprintf (stderr, "Unsupported stop bits\n");
      return (FALSE);
    }
  /* Set input parity option */
  if (parity != 'n')
    options.c_iflag |= INPCK;
  tcflush (fd, TCIFLUSH);
  options.c_cc[VTIME] = 150;
  options.c_cc[VMIN] = 0;	/* Update the options and do it NOW */
  if (tcsetattr (fd, TCSANOW, &options) != 0)
    {
      perror ("SetupSerial 3");
      return (FALSE);
    }
  return (TRUE);
}

char buffer[256][256];

int Readfile()
 {
  int ret = 0;
  FILE *stream;
  int i;
  stream = fopen("/tmp/lcdtemp", "r"); //led file is save string 
  while((fgets(buffer[ret], 256, stream)) != NULL)
  {
	i = strlen(buffer[ret]);
	//printf("i=%d\n",i);
        buffer[ret][i-1] = '\0'; //save to two-dimensional array
	ret++;
  }
  fclose(stream);
  return ret;
}

int Uart_send(int fd, char *send_buf, int data_len)
{
  int ret;

  ret = write(fd, send_buf, data_len);
  if (data_len == ret) {
    return ret;
  } else {
    tcflush(fd, TCOFLUSH);
    return FALSE;
  }
}
/*

*/

void Print_string(int fd, char *ptr)
{
  int i = 0;
  char e[] = {0x0d}; // "\n" 
  while(*ptr != '\0')
  {
    if (i == 16)
    {
      Uart_send(fd, e, 1);
      usleep(2000); //"\n" must sleep 2000us
    }
    Uart_send(fd, ptr, 1);
    *ptr++;
    i++;
    usleep(500); // send a string  sleep 500us
  }
}

int
main ()
{
  //printf ("This program updates last time at %s   %s\n", __TIME__, __DATE__);
  //printf ("STDIO COM1\n");
  int fd;
  int ret;
  int i;
  fd = open ("/dev/ttyS1", O_RDWR);
  if (fd == -1)
    {
      perror ("serialport error\n");
    }
  else
    {
     // printf ("open ");
      //printf ("%s", ttyname (fd));
      //printf (" succesfully\n");
    }

  set_speed (fd, 115200);
  if (set_Parity (fd, 8, 1, 'N') == FALSE)
    {
      printf ("Set Parity Error\n");
      exit (0);
    }
  ret = Readfile();
//printf("ret=%d\n", ret);
	//char hello[] = "asdfghjkl";
	//write(fd, hello, sizeof(hello));
  char buf[255];
  fd_set rd;
  int nread = 0;
  int j = 0;
  char CL[] = {0XFF, '\0'}; //clear screen
  char DL[] = {0X0D, '\0'}; //"\d" A newline
  while(1)
  {
	Print_string(fd, CL);
    usleep(2000);
	//printf("1111111111111111111111\n");
     Print_string(fd, buffer[0]); // The first and second lines of fixed display file
     Print_string(fd, DL);
     usleep(2000);
     Print_string(fd, buffer[1]);
	i = 1;
  	FD_ZERO(&rd);
	FD_SET(fd, &rd);
	while(FD_ISSET(fd, &rd))
	{
	//printf("22222222222222222222222\n");
		if(select(fd+1, &rd, NULL,NULL,NULL) < 0)
		{
			perror("select error\n");
		}
		else
		{
			while((nread = read(fd, buf, sizeof(buf))) > 0)
			{
				//printf("333333333333333333333\n");
					switch(buf[0]){
						case '2': //
						i++;
						//printf("i=%d\n", i);
						if(ret == i){
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[i-1]);
							Print_string(fd, DL);
							usleep(2000);
     						Print_string(fd, buffer[i]);
						}else if (i > ret){
							i = ret;
							break;
						}else{
							j = (i+1)%ret;
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[i]);
							Print_string(fd, DL);
							usleep(2000);
     							Print_string(fd, buffer[j]);
						}
  						break;
			 			case '8':
						i--;
						//printf("i=%d\n", i);
						if(0 == i){
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[0]);
							Print_string(fd, DL);
							usleep(2000);
							Print_string(fd, buffer[1]);
						}else if(i < 0) {
							i = 1;
							break;
						}else{
							j = (i+1)%ret;
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[i]);
							Print_string(fd, DL);
							usleep(2000);
     						Print_string(fd, buffer[j]);
						}
						break;
						case '4':
							i -= 2;
						//printf("i=%d\n", i);
						if(1 == i){
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[0]);
							Print_string(fd, DL);
							usleep(2000);
							Print_string(fd, buffer[1]);
						} else if(i < 0){
							i = 1;
							break;
						}else{
							j = (i+ret-1)%ret;
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[j]);
							Print_string(fd, DL);
							usleep(2000);
     						Print_string(fd, buffer[i]);
						}
						break;
						case '6':
							i += 2;
						//printf("i=%d\n", i);
						if(i > ret){
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[ret-1]);
							Print_string(fd, DL);
							usleep(2000);
							Print_string(fd, buffer[ret]);
							
						} else if(i > ret) {
							i = ret;
							break;
						}else{
							j = (i+ret-1)%ret;
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[j]);
							Print_string(fd, DL);
							usleep(2000);
							Print_string(fd, buffer[i]);
								
						}
						break;
						case '5':
							Print_string(fd, CL);
							usleep(2000);
							Print_string(fd, buffer[0]);
							Print_string(fd, DL);
							usleep(2000);
							Print_string(fd, buffer[1]);
							break;
									

						default:
						break;
					}
				printf("nread = %d,%s\n",nread, buf);
				//printf("test\n");
				memset(buf, 0 , sizeof(buf));
			}
		}
	}
  }
  close (fd);
  return 0;
}
